package com.loit.common.insertSql;

import com.loit.common.insertSql.dto.ColumnInfoDTO;
import com.loit.common.insertSql.dto.LineInfoDTO;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.common.utils.excel.ImportExcel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.http.entity.ContentType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@Slf4j
public class GeneratorInsertSqlScript {


    private static Logger logger = LoggerFactory.getLogger(GeneratorInsertSqlScript.class);

    //TODO  1：事件，2：部件
    public static final String CASE_TYPE = "2";
    public static final String FILE_NAME = "loit_bc_process_config_node数据初始化.xlsx";

    public static Map<String, String> columnNameMap = new LinkedHashMap<>();

    public static Map<String, String> columnTypeMap = new LinkedHashMap<>();

    public static int maxIndex = 100;

    public static String tableName;

    protected static String root_path = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources";


    public static void main(String[] args) {


        try {

            String filePathStr = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\" + FILE_NAME;

            File pdfFile = new File(filePathStr);
            FileInputStream fileInputStream = new FileInputStream(pdfFile);
            MultipartFile multipartFile = new MockMultipartFile(pdfFile.getName(), pdfFile.getName(),

                    ContentType.APPLICATION_OCTET_STREAM.toString(), fileInputStream);
            ImportExcel ei = new ImportExcel(multipartFile, 0, 0);
            List<LineInfoDTO> commandManualDTOList = ei.getDataList(LineInfoDTO.class);

            if (ListUtil.isEmpty(commandManualDTOList)) {
                return;
            }

            tableName = commandManualDTOList.get(0).getCol1();
            LineInfoDTO columnLineInfo = commandManualDTOList.get(1);
            for (int i = 0; i < 24; i++) {
                int index = i + 1;
                String colName = BeanUtils.getProperty(columnLineInfo, "col" + index);
                columnNameMap.put(i + "", colName);
            }

            LineInfoDTO columnTypeInfo = commandManualDTOList.get(2);

            for (int colIndex = 0; colIndex < 24; colIndex++) {
                int index = colIndex + 1;
                String colName = BeanUtils.getProperty(columnTypeInfo, "col" + index);
                if (StringUtils.isNotEmpty(colName)) {
                    columnTypeMap.put(colIndex + "", colName);
                }

            }


            for (int valueIndex = 0; valueIndex < commandManualDTOList.size(); valueIndex++) {
                if (valueIndex <= 2) {
                    continue;
                }

                List<ColumnInfoDTO> columnInfoList = new ArrayList<>();
                LineInfoDTO columnValueInfo = commandManualDTOList.get(valueIndex);


                boolean isDataLine = true;
                for (int colIndex = 0; colIndex < 24; colIndex++) {

                    int index = colIndex + 1;
                    String colValue = BeanUtils.getProperty(columnValueInfo, "col" + index);
                    if (index == 1 && StringUtils.isEmpty(colValue)) {
                        isDataLine = false;
                        break;
                    }

                    String columnName = columnNameMap.get(colIndex + "");
                    if (StringUtils.isEmpty(columnName)) {
                        continue;
                    }

                    ColumnInfoDTO columnInfoDTO = new ColumnInfoDTO();
                    columnInfoDTO.setColumnValue(colValue);
                    columnInfoDTO.setTableName(tableName);
                    columnInfoDTO.setColumnName(columnName);
                    columnInfoDTO.setColumnType(columnTypeMap.get(colIndex + ""));
                    columnInfoList.add(columnInfoDTO);
                }


                if (isDataLine) {
                    createInsertSql(columnInfoList);
                }


            }


        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }


    public static void createInsertSql(List<ColumnInfoDTO> columnInfoList) {

        StringBuffer sql = new StringBuffer("INSERT INTO `");
        sql.append(tableName);
        sql.append("` (");
        sql.append(columNameStr(columnInfoList));
        sql.append(") VALUES (");
        sql.append(columnValueStr(columnInfoList));
        sql.append(");");

        System.out.println(sql.toString());

    }


    public static String columNameStr(List<ColumnInfoDTO> columnInfoList) {

        StringBuilder resultBuilder = new StringBuilder();
        for (ColumnInfoDTO columnInfoDTO : columnInfoList) {
            resultBuilder.append("`" + columnInfoDTO.getColumnName() + "`").append(", ");
        }

        String result = resultBuilder.toString();
        if (result.endsWith(", ")) {
            result = result.substring(0, result.length() - 2);
        }

        return result;

    }

    public static String columnValueStr(List<ColumnInfoDTO> columnInfoList) {

        StringBuilder resultBuilder = new StringBuilder();
        for (ColumnInfoDTO columnInfoDTO : columnInfoList) {
            String columnType = columnInfoDTO.getColumnType();
            String columnValue = columnInfoDTO.getColumnValue();

            if (StringUtils.isEmpty(columnValue)) {
                resultBuilder.append("NULL").append(", ");
                continue;
            }

            if (StringUtils.isEmpty(columnType)) {
                resultBuilder.append("'" + columnInfoDTO.getColumnValue() + "'").append(", ");
            } else {
                resultBuilder.append(columnInfoDTO.getColumnType() + "'" + columnInfoDTO.getColumnValue() + "'").append(", ");
            }
        }

        String result = resultBuilder.toString();
        if (result.endsWith(", ")) {
            result = result.substring(0, result.length() - 2);
        }

        return result;

    }

}
