package com.loit.common.script;

import com.loit.common.script.dto.BuildEnvEnum;
import com.loit.common.script.dto.SqlDDLDataDTO;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.common.utils.excel.ImportExcel;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.entity.ContentType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

@Slf4j
public class GeneratorSqlDdlScript {

    protected static String root_path = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\bin";

    /**
     * 构建环境 生产或者 准生产
     * TODO 构建前需修改环境
     */
    private static BuildEnvEnum buildEnvEnum = BuildEnvEnum.PRE_PRODUCT;

    protected static String root_path_jenkins_full = root_path + "\\jenkinsDeploy";


    public static void main(String[] args) {

        try {
            //String filePathStr = Thread.currentThread().getContextClassLoader().getResource("deployInfo.xlsx").getPath();
            String filePathStr = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\createSql.xlsx";

            File pdfFile = new File(filePathStr);
            FileInputStream fileInputStream = new FileInputStream(pdfFile);
            MultipartFile multipartFile = new MockMultipartFile(pdfFile.getName(), pdfFile.getName(),

                    ContentType.APPLICATION_OCTET_STREAM.toString(), fileInputStream);
            ImportExcel ei = new ImportExcel(multipartFile, 0, 0);
            List<SqlDDLDataDTO> sqlDDLDataDTOInitList = ei.getDataList(SqlDDLDataDTO.class);

            if (ListUtil.isEmpty(sqlDDLDataDTOInitList)) {
                return;
            }

            String tableChineseName = "";
            String tableEnglishName = "";

            // 是否自增
            boolean autoIncrement = false;

            List<SqlDDLDataDTO> ddlInfoList = new ArrayList<>();
            for (SqlDDLDataDTO grayVersionDataDTO : sqlDDLDataDTOInitList) {

                String fieldName = grayVersionDataDTO.getFieldName();
                if (StringUtils.isEmpty(fieldName)) {
                    continue;
                }

                ddlInfoList.add(grayVersionDataDTO);

                if (StringUtils.isNotEmpty(grayVersionDataDTO.getTableChineseName())) {
                    tableChineseName = grayVersionDataDTO.getTableChineseName();
                }

                if (StringUtils.isNotEmpty(grayVersionDataDTO.getTableEnglishName())) {
                    tableEnglishName = grayVersionDataDTO.getTableEnglishName();
                }


                if (StringUtils.isNotEmpty(grayVersionDataDTO.getAutoIncrement())) {
                    autoIncrement = "Y".equals(grayVersionDataDTO.getAutoIncrement());
                }

            }


            StringBuffer ddl = new StringBuffer();
            ddl.append("DROP TABLE IF EXISTS " + "`" + tableEnglishName + "`;" + "\r\n");

            ddl.append("CREATE TABLE " + "`" + tableEnglishName + "` (" + "\r\n");


            Iterator<SqlDDLDataDTO> iterator = ddlInfoList.iterator();
            while (iterator.hasNext()) {
                SqlDDLDataDTO sqlDDLDataDTO = iterator.next();

                String fieldName = sqlDDLDataDTO.getFieldName();
                String fieldType = sqlDDLDataDTO.getFieldType();
                String fieldLength = sqlDDLDataDTO.getFieldLength();
                if (StringUtils.isEmpty(fieldLength)) {
                    fieldLength = "0";
                }

                String fieldComment = sqlDDLDataDTO.getFieldComment();
                String ifKey = sqlDDLDataDTO.getIfKey();
                String ifnullValue = sqlDDLDataDTO.getIfnullValue();
                String defaultV = sqlDDLDataDTO.getDefaultVal();

                String defaultVal = "";

                if (fieldType.toLowerCase().contains("varchar") && StringUtils.isNotEmpty(defaultV)) {
                    defaultVal = "DEFAULT '" + defaultV + "'";
                } else if (fieldType.toLowerCase().contains("int") && StringUtils.isNotEmpty(defaultV)) {
                    defaultVal = "DEFAULT " + defaultV;
                } else if (!"N".equals(ifnullValue)) {
                    defaultVal = "DEFAULT NULL ";
                }

                ddl.append(
                        "`" + fieldName + "` "
                                + fieldType
                                + (!"0".equals(fieldLength) ? "(" + fieldLength + ")" : "")
                                + ("Y".equals(autoIncrement) ? " PRIMARY KEY " : "")
                                + ("Y".equals(autoIncrement) ? " AUTO_INCREMENT " : "")
                                + ("N".equals(ifnullValue) ? " NOT NULL " : "")
                                + (StringUtils.isNotEmpty(defaultVal) ? " " +defaultVal + " " : "")
                                + " COMMENT '" + fieldComment + "'"
                                + (iterator.hasNext() ? ",\r\n" : "\r\n")
                );
//                if ("Y".equals(ifKey)) {
//                    autoIncrement = true;
//                }
            }

            if (ddl.toString().endsWith(",\r\n")) {
                ddl = ddl.deleteCharAt(ddl.length() - 3);
//                ddl.append("\r\n");
            }
            ddl.append(") ENGINE=InnoDB " + (autoIncrement ? "AUTO_INCREMENT=1" : "") + " DEFAULT CHARSET=utf8 "
                    + (!"".equals(tableChineseName) ? "COMMENT = '" + tableChineseName + "'" : "") + ";\r\n");
            ddl.append("-- --------------------------------------------------------------------------------\r\n");
            System.out.println(ddl.toString());


        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }


}
